@@ -21,6 +21,7 @@ BACKGROUND = "#FFFFFF"
 COLUMNS_ACC = ["Name", "Description", "Quantity","Grouping", "Position_1", "Position_2", "Position_3", "Orientation_1", "Orientation_2", "Orientation_3"]
 COLUMNS_CHN = ["Name", "Description","Quantity","Grouping","Position_1", "Position_2", "Position_3", "Direction_1", "Direction_2","Direction_3"]
 
+
 class view3D():
     """
     A 3D display where structure, impacts, accelerometer and channels can be quickly displayed. Additionaly, all objects
@@ -55,6 +56,8 @@ class view3D():
         if show_axes:
             self.plot.add_axes(labels_off=True)
 
+        self.plot.enable_parallel_projection()
+
         # Static global variables
         self.global_acc = []
         self.acc_visible = False
@@ -92,6 +95,66 @@ class view3D():
         self.animate_toolbar = self.plot.app_window.addToolBar('Animate Modeshape')
         self.animate_clear_toolbar = self.plot.app_window.addToolBar('Clear Modeshape')
 
+        # Temp
+        self._points = []
+        self._directions = []
+        self.scale = 10
+        self.toggle = None
+
+    @property
+    def points(self):
+        """To access all the points when done."""
+        return self._points
+    
+    @property
+    def directions(self):
+        """To access all the directions when done."""
+        return self._directions
+
+    def toggle_fun(self):
+        if self.toggle == None:
+            self.plot.track_click_position(self, side='right')
+
+        
+    def __call__(self, *args):
+        """Callback function to access the location."""
+        # picked point
+        picked_pt = np.array(self.plot.pick_mouse_position())
+        direction = picked_pt - self.plot.camera_position[0]
+        direction = direction / np.linalg.norm(direction)
+
+        # define ray 
+        start = picked_pt - 1000 * direction
+        end = picked_pt + 10000 * direction
+        # ray tracing 
+        point, ix = self.mesh.ray_trace(start, end, first_point=True)
+        if len(point) > 0:
+            normal = self.mesh.cell_normals[int(ix)]
+
+            # append points
+            self._points.append(point)
+            self._directions.append(np.array(-normal))
+            
+            # Define callback function
+            if self.toggle == "impact":
+                self.imp_callback(point,np.array(-normal))
+            elif self.toggle == "acc":
+                rot = rotation_matrix_from_vectors(np.array(-normal), np.array([0.,0.,1.]))
+
+                local_orientation = np.asarray([[1, 0, 0],
+                                             [0, 1, 0],
+                                             [0, 0, 1]])
+
+                local_orientation = (rot @ (local_orientation))
+                r = R.from_matrix(local_orientation)
+                orientation = np.asarray(r.as_euler('xyz', degrees=True))
+                # just added static size of the accelerometer
+                point +=  np.array(+normal) / 2 * 10
+                self.acc_callback(point,orientation=orientation)
+            else:
+                pass
+
+
 
     def add_modeshape(self,dict_animation,run_animation = False,add_note = False):
         """
@@ -433,7 +496,7 @@ class view3D():
             rot = r.as_matrix()
 
         self.add_accelerometer(acc)
-        _gg = DynamicPosition(acc, self.plot, i, mesh=self.mesh, size=10,rot = rot,fixed_rotation = fixed_rotation)
+        _gg = DynamicPosition(acc, self.plot, i, mesh=self.mesh, size=10,rot = rot,fixed_rotation = fixed_rotation,snap_outward = True)
         self.plot.add_sphere_widget(_gg.callback, center=_gg.points, color=["k", "r", "g", "b"], radius=10 / 15)
         _gg.translate(point)
         _gg.turn_on = True
@@ -455,16 +518,20 @@ class view3D():
         """
 
         self.mesh = mesh
+        self.mesh.compute_normals(auto_orient_normals=True, inplace=True)
+
+        self.toggle_fun()
+        self.toggle = "acc"
+
 
         if isinstance(predefined, pd.DataFrame):
             for i, row in predefined.iterrows():
-                point = [row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale]
-                orientation = [row["Orientation_1"], row["Orientation_2"], row["Orientation_3"]]
+                point = np.asarray([row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale])
+                orientation = np.asarray([row["Orientation_1"], row["Orientation_2"], row["Orientation_3"]])
                 self.acc_callback(point, orientation=orientation,fixed_rotation = fixed_rotation)
+        
+        self.plot.add_text('Use right mouse click to add an accelerometer', color="k", font="times",font_size = 10, name="text")
 
-        self.plot.enable_point_picking(callback=self.acc_callback, color="r", show_message="", show_point=False)
-        self.plot.add_text("Press P too add an accelerometer (hold down letter T to disable snapping to mesh).",
-                           font_size=10, color="k", font="times", name="text")
 
     def imp_callback(self,point, direction = None,fixed_rotation = None):
         """
@@ -485,11 +552,10 @@ class view3D():
             rot = np.diag([1]*3)
         else:
             imp, _ = self.add_impact([size/2, size/2, size/2],direction, size=10)
-
-            #rot = np.diag([1]*3)
             rot = rotation_matrix_from_vectors(direction,[0, 0, 1]).T
 
-        _gg = DynamicPosition([imp], self.plot, i, mesh=self.mesh, size=10,rot = rot,snap_outward = False, fixed_rotation = fixed_rotation)
+        
+        _gg = DynamicPosition([imp], self.plot, i, mesh=self.mesh, size=10,rot = rot,snap_outward = False, fixed_rotation = fixed_rotation, toggle = "impact")
         self.plot.add_sphere_widget(_gg.callback, center=_gg.points, color=["k", "r", "g", "b"], radius=10 / 15)
         _gg.translate(point)
         _gg.turn_on = True
@@ -511,15 +577,20 @@ class view3D():
         """
 
         self.mesh = mesh
+        self.mesh.compute_normals(auto_orient_normals=True, inplace=True)
+
+        self.toggle_fun()
+        self.toggle = "impact"
+
 
         if isinstance(predefined, pd.DataFrame):
             for i, row in predefined.iterrows():
-                point = [row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale]
-                direction = [row["Direction_1"], row["Direction_2"], row["Direction_3"]]
+                point = np.asarray([row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale])
+                direction = np.asarray([row["Direction_1"], row["Direction_2"], row["Direction_3"]])
                 self.imp_callback(point, direction=direction,fixed_rotation = fixed_rotation)
 
-        self.plot.enable_point_picking(callback=self.imp_callback, color="r", show_message="", show_point=False)
-        self.plot.add_text("Press P too add an impact (hold down letter T to disable snapping to mesh).", font_size = 10,color = "k",font  = "times",name = "text")
+        self.plot.add_text('Use right mouse click to add an impact', color="k", font="times",font_size = 10, name="text")
+
 
     def vp_callback(self,point,fixed_rotation = None):
         """
@@ -993,7 +1064,7 @@ class DynamicPosition():
     :type fixed_rotation: float
     """
 
-    def __init__(self, objects, p, N, mesh=None, snap_outward=True, size=1, rot = np.diag([1]*3), fixed_rotation = None):
+    def __init__(self, objects, p, N, mesh=None, snap_outward=True, size=1, rot = np.diag([1]*3), fixed_rotation = None,toggle = "acc"):
         # set size and static points
         self.size = size
         self.points = np.array([[size / 2, size / 2, size / 2],
@@ -1060,6 +1131,7 @@ class DynamicPosition():
 
         # define display
         self.p = p
+        self.toggle = toggle
 
     def get_pos_orient(self, euler_angles=False, one_dir=None, eps=1e-10,scale = 1):
         """
@@ -1107,62 +1179,55 @@ class DynamicPosition():
         :type snap: bool, optional
         """
 
-        # definest rays to find intersection with the supplied mesh
-        point1x = point + self.local_rays[:, 0]
-        point2x = point + self.local_rays[:, 3]
+		# option B
+        if snap:
+            direction = np.asarray(point) - np.asarray(self.p.camera_position[0])
+            direction = direction / np.linalg.norm(direction)
+            start = point - 1000 * direction
+            end = point + 10000 * direction
+            points, ind = self.mesh.ray_trace(start, end, first_point=True)
 
-        point1y = point + self.local_rays[:, 1]
-        point2y = point + self.local_rays[:, 4]
-
-        point1z = point + self.local_rays[:, 2]
-        point2z = point + self.local_rays[:, 5]
-
-        # performs ray trace in three direction
-        points_x, ind_x = self.mesh.ray_trace(point1x, point2x)
-        points_y, ind_y = self.mesh.ray_trace(point1y, point2y)
-        points_z, ind_z = self.mesh.ray_trace(point1z, point2z)
-
-        # stacks all the ray intersections
-        points = np.vstack([points_x, points_y, points_z])
-        ind = np.hstack([ind_x, ind_y, ind_z])
-
-        # default option - no rotation
-        rot = np.diag([1, 1, 1])
-
-        # if there is an intersection and if "t" is not pressed go forward
-        if points.size != 0 and not (kb.is_pressed('t')):
-            list_ind = []
-            # go through all the intersections
-            for i in range(len(points)):
-                _point = points[i]
-                p1 = _point
-                p2 = point
-                gg = np.sqrt(((p1[0] - p2[0]) ** 2) + ((p1[1] - p2[1]) ** 2) + ((p1[2] - p2[2]) ** 2))
-
-                list_ind.append(gg)
-
-            # find the closest to the box center
-            _sel = np.argmin(list_ind)
-
-            # find the nearest normal
-            v2 = self.mesh.cell_normals[int(ind[_sel])]
-            th = []
-            for _loc in self.local_normals.T:
-                th.append(angle_between(_loc, v2))
-            closest_orient = self.local_normals.T[np.argmin(th)]
-
-            # find orientation between box orientation and cell normal
-            f = self.mesh.cell_normals[int(ind[_sel])]
-            t = closest_orient + np.random.random(3) / 1e20
-
-            # push box 0.5 away from the normal
+            # fast upgrade to option B
             if self.snap_outward:
-                point = points[_sel] + f / 2 * self.size
-            else:
-                point = points[_sel]
+                f = self.mesh.cell_normals[int(ind)]
+                point = points - f / 2 * self.size
+                direction = np.asarray(point) - np.asarray(self.p.camera_position[0])
+                direction = direction / np.linalg.norm(direction)
+                start = point - 1000 * direction
+                end = point + 10000 * direction
+                points, ind = self.mesh.ray_trace(start, end, first_point=True)
 
-            # define rotational matrix to allign with the surface normal
-            rot = rotation_matrix_from_vectors(t, f)
+            # default option - no rotation
+            rot = np.diag([1., 1., 1.])
+
+            # if there is an intersection and if "t" is not pressed go forward
+            if points.size != 0 and not (kb.is_pressed('t')):
+                # find the nearest normal
+                v2 = self.mesh.cell_normals[int(ind)]
+                th = []
+                for _loc in self.local_normals.T:
+                    th.append(angle_between(_loc, v2))
+                closest_orient = self.local_normals.T[np.argmin(th)]
+
+                # find orientation between box orientation and cell normal
+                f = self.mesh.cell_normals[int(ind)]
+                t = closest_orient #+ np.random.random(3) / 1e20
+                if self.toggle == "impact":
+                    closest_orient = self.local_normals.T[2] # always Z axis
+                    t = closest_orient #+ np.random.random(3) / 1e20
+                    f = -1*self.mesh.cell_normals[int(ind)]
+
+                # push box 0.5 away from the normal
+                if self.snap_outward:
+                    point = points + f / 2 * self.size
+                else:
+                    point = points
+
+                # define rotational matrix to allign with the surface normal
+                #if 't' not in vars() or  'f' not in vars():
+                #    rot = rotation_matrix_from_vectors(np.asarray([0.,0.,1.]).T, np.asarray([0.,0.,1.]).T)
+                #else:
+                rot = rotation_matrix_from_vectors(t, f)
 
         # move everything to a new location
         _new = point - self.box.center_of_mass()
@@ -1229,7 +1294,7 @@ class DynamicPosition():
                 for item in self.objects:
                     item.points = (rot @ (item.points - _new).T).T + _new
 
-                    # orient the local csys of accelerometer with the new rotation
+                # orient the local csys of accelerometer with the new rotation
                 self.local_orientation = (rot @ (self.local_orientation))
                 self.local_normals = rot @ self.local_normals
                 self.local_rays = rot @ self.local_rays
